@@ -1,7 +1,7 @@
 /*
  * Copyright 2005-2006 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/portage-utils/main.c,v 1.135 2007/03/16 21:31:11 solar Exp $
+ * $Header: /var/cvsroot/gentoo-projects/portage-utils/main.c,v 1.140 2007/04/18 17:43:08 vapier Exp $
  *
  * Copyright 2005-2006 Ned Ludd        - <solar@gentoo.org>
  * Copyright 2005-2006 Mike Frysinger  - <vapier@gentoo.org>
@@ -281,8 +281,7 @@ static char *remove_extra_space(char *str)
 	if (str == NULL)
 		return NULL;
 	len = strlen(str);
-	buf = xmalloc(len+1);
-	memset(buf, 0, len+1);
+	buf = xzalloc(len+1);
 	for (p = str; *p != 0; ++p) {
 		if (!isspace(*p)) c = *p; else {
 			if (c == ' ') continue;
@@ -434,9 +433,8 @@ char *strincr_var(const char *name, char *s, char *value, const size_t value_len
 	char buf[BUFSIZ];
 	char *p;
 
-
 	if ((strlen(value) + 1 + strlen(s)) >= value_len)
-		errf("%s will exceed max length value of %d with a size of %d", name, value_len, (strlen(value) + 1 + strlen(s)));
+		errf("%s will exceed max length value of %zi with a size of %zi", name, value_len, (strlen(value) + 1 + strlen(s)));
 
 	strncat(value, " ", value_len);
 	strncat(value, s, value_len);
@@ -509,7 +507,10 @@ void initialize_portage_env(void)
 	f = 0;
 	if (readlink("/etc/make.profile", profile, sizeof(profile)) == -1)
 		strcpy(profile, "/etc/make.profile");
-	chdir("/etc");
+	if (profile[0] != '/') {
+		memmove(profile+5, profile, strlen(profile));
+		memcpy(profile, "/etc/", 5);
+	}
 	do {
 		if (f == 0)
 			snprintf(portage_file, sizeof(portage_file), "%s/make.defaults", profile);
@@ -732,7 +733,11 @@ const char *initialize_flat(int cache_type)
 
 	warn("Finished %u entries in %d.%06d seconds", count, secs, frac);
 	if (secs > 100)
+#ifdef __linux__
 		warn("You should consider a faster file system such as reiserfs for PORTDIR='%s'", portdir);
+#else
+		warn("You should consider using the noatime mount option for PORTDIR='%s' if it's not already enabled", portdir);
+#endif
 ret:
 	return cache_file;
 }
@@ -793,8 +798,7 @@ portage_cache *cache_read_file(const char *file)
 	if (fstat(fileno(f), &s) != 0)
 		goto err;
 	len = sizeof(*ret) + s.st_size + 1;
-	ret = xmalloc(len);
-	memset(ret, 0x00, len);
+	ret = xzalloc(len);
 #if 0
 {	/* This is the expected format of portage-2.0 cache code NAME=var */
 	/* at any time ferringb might decide to add new entries. CDEPEND= will be no more */
