@@ -1,7 +1,7 @@
 /*
  * Copyright 2005-2006 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/portage-utils/qcache.c,v 1.22 2007/02/20 23:14:04 solar Exp $
+ * $Header: /var/cvsroot/gentoo-projects/portage-utils/qcache.c,v 1.29 2007/04/18 18:47:18 solar Exp $
  *
  * Copyright 2006 Thomas A. Cort - <tcort@gentoo.org>
  */
@@ -48,7 +48,7 @@ static const char *qcache_opts_help[] = {
 	COMMON_OPTS_HELP
 };
 
-static const char qcache_rcsid[] = "$Id: qcache.c,v 1.22 2007/02/20 23:14:04 solar Exp $";
+static const char qcache_rcsid[] = "$Id: qcache.c,v 1.29 2007/04/18 18:47:18 solar Exp $";
 #define qcache_usage(ret) usage(ret, QCACHE_FLAGS, qcache_long_opts, qcache_opts_help, lookup_applet_idx("qcache"))
 
 /********************************************************************/
@@ -230,8 +230,7 @@ int read_keywords(char *s, int *keywords)
  * ERR:
  *  -1 is returned if the file cannot be read.
  */
-inline unsigned int qcache_count_lines(char *filename);
-inline unsigned int qcache_count_lines(char *filename)
+static unsigned int qcache_count_lines(char *filename)
 {
 	unsigned int count, fd;
 	char c;
@@ -272,8 +271,7 @@ char **qcache_read_lines(char *filename)
 		return NULL;
 
 	len   = sizeof(char*) * (num_lines + 1);
-	lines = xmalloc(len);
-	memset(lines, 0, len);
+	lines = xzalloc(len);
 
 	if ((fd = open(filename, O_RDONLY)) != -1) {
 		for (i = 0; i < num_lines; i++) {
@@ -284,8 +282,7 @@ char **qcache_read_lines(char *filename)
 				count++;
 			lseek(fd, (lseek(fd, 0, SEEK_CUR) - count - 1), SEEK_SET);
 
-			lines[i] = xmalloc(sizeof(char) * (count+1));
-			memset(lines[i], 0, count+1);
+			lines[i] = xzalloc(sizeof(char) * (count+1));
 
 			/* copy the line into lines[i] */
 			read(fd, lines[i], count);
@@ -345,8 +342,7 @@ portage_cache *qcache_read_cache_file(const char *filename)
 	}
 
 	len = sizeof(*ret) + s.st_size + 1;
-	ret = xmalloc(len);
-	memset(ret, 0x00, len);
+	ret = xzalloc(len);
 
 	while ((fgets(buf, sizeof(buf), f)) != NULL) {
 		if ((ptr = strrchr(buf, '\n')) != NULL)
@@ -516,10 +512,10 @@ int qcache_traverse(void (*func)(qcache_data*))
 	int i, j, k, len, num_cat, num_pkg, num_ebuild;
 	struct direct **categories, **packages, **ebuilds;
 
-	len = asprintf(&catpath, "%s%s", QCACHE_EDB, portdir);
+	xasprintf(&catpath, "%s%s", QCACHE_EDB, portdir);
 
 	if (-1 == (num_cat = scandir(catpath, &categories, qcache_file_select, alphasort))) {
-		err("%s %s", catpath, strerror(errno));
+		errp("%s", catpath);
 		free(catpath);
 	}
 
@@ -528,10 +524,10 @@ int qcache_traverse(void (*func)(qcache_data*))
 
 	/* traverse categories */
 	for (i = 0; i < num_cat; i++) {
-		len = asprintf(&pkgpath, "%s/%s", portdir, categories[i]->d_name);
+		xasprintf(&pkgpath, "%s/%s", portdir, categories[i]->d_name);
 
 		if (-1 == (num_pkg = scandir(pkgpath, &packages, qcache_file_select, alphasort))) {
-			warn("%s %s", pkgpath, strerror(errno));
+			warnp("Found a cache dir, but unable to process %s", pkgpath);
 			free(categories[i]);
 			free(pkgpath);
 			continue;
@@ -551,12 +547,11 @@ int qcache_traverse(void (*func)(qcache_data*))
 		/* traverse packages */
 		for (j = 0; j < num_pkg; j++) {
 			len = sizeof(char) * (strlen(portdir) + strlen("/") + strlen(categories[i]->d_name) + strlen("/") + strlen(packages[j]->d_name) + 1);
-			ebuildpath = xmalloc(len);
-			memset(ebuildpath, 0, len);
+			ebuildpath = xzalloc(len);
 			snprintf(ebuildpath, len, "%s/%s/%s", portdir, categories[i]->d_name, packages[j]->d_name);
 
 			if (-1 == (num_ebuild = scandir(ebuildpath, &ebuilds, qcache_ebuild_select, qcache_vercmp))) {
-				warn("%s %s", ebuildpath, strerror(errno));
+				warnp("%s", ebuildpath);
 				free(packages[i]);
 				free(pkgpath);
 				continue;
@@ -578,26 +573,29 @@ int qcache_traverse(void (*func)(qcache_data*))
 			/* traverse ebuilds */
 			for (k = 0; k < num_ebuild; k++) {
 				len = sizeof(char) * (strlen(catpath) + strlen("/") + strlen(categories[i]->d_name) + strlen("/") + strlen(ebuilds[k]->d_name) + 1);
-				cachepath = xmalloc(len);
-				memset(cachepath, 0, len);
+				cachepath = xzalloc(len);
 				snprintf(cachepath, len, "%s/%s/%s", catpath, categories[i]->d_name, ebuilds[k]->d_name);
 				cachepath[len-8] = '\0'; /* remove ".ebuild" */
 
-				data.cache_data = qcache_read_cache_file(cachepath);
 				data.category = categories[i]->d_name;
 				data.package = packages[j]->d_name;
 				data.ebuild = ebuilds[k]->d_name;
 				data.cur = k + 1;
 				data.num = num_ebuild;
+				data.cache_data = qcache_read_cache_file(cachepath);
 
-				/* is this the last ebuild? */
-				if (i+1 == num_cat && j+1 == num_pkg && k+1 == num_ebuild)
-					qcache_last = 1;
+				if (data.cache_data != NULL) {
+					/* is this the last ebuild? */
+					if (i+1 == num_cat && j+1 == num_pkg && k+1 == num_ebuild)
+						qcache_last = 1;
 
-				if (!qcache_skip)
-					func(&data);
+					if (!qcache_skip)
+						func(&data);
+
+					qcache_free_data(data.cache_data);
+				} else
+					warn("unable to read cache '%s'\n\tperhaps you need to `emerge --metadata` or `emerge --regen` ?", cachepath);
 
-				qcache_free_data(data.cache_data);
 				free(ebuilds[k]);
 				free(cachepath);
 			}
@@ -772,12 +770,11 @@ void qcache_stats(qcache_data *data)
 			architectures++;
 
 		len = sizeof(char) * (strlen(QCACHE_EDB) + strlen(portdir) + 1);
-		catpath = xmalloc(len);
-		memset(catpath, 0, len);
+		catpath = xzalloc(len);
 		snprintf(catpath, len, "%s%s", QCACHE_EDB, portdir);
 
 		if (-1 == (numcat = scandir(catpath, &categories, qcache_file_select, alphasort))) {
-			err("%s %s", catpath, strerror(errno));
+			errp("%s", catpath);
 			free(catpath);
 		}
 
@@ -927,9 +924,8 @@ int qcache_init()
 	unsigned int len;
 
 	len      = sizeof(char) * (strlen(portdir) + strlen("/profiles/arch.list") + 1);
-	filename = xmalloc(len);
+	filename = xzalloc(len);
 
-	memset(filename, 0, len);
 	snprintf(filename, len, "%s/profiles/arch.list", portdir);
 
 	if (NULL == (archlist = qcache_read_lines(filename))) {
