@@ -1,7 +1,7 @@
 /*
  * Copyright 2005-2006 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/portage-utils/qpkg.c,v 1.20 2006/06/05 12:06:00 solar Exp $
+ * $Header: /var/cvsroot/gentoo-projects/portage-utils/qpkg.c,v 1.21 2007/04/18 18:20:47 vapier Exp $
  *
  * Copyright 2005-2006 Ned Ludd        - <solar@gentoo.org>
  * Copyright 2005-2006 Mike Frysinger  - <vapier@gentoo.org>
@@ -20,7 +20,7 @@ static const char *qpkg_opts_help[] = {
 	"alternate package directory",
 	COMMON_OPTS_HELP
 };
-static const char qpkg_rcsid[] = "$Id: qpkg.c,v 1.20 2006/06/05 12:06:00 solar Exp $";
+static const char qpkg_rcsid[] = "$Id: qpkg.c,v 1.21 2007/04/18 18:20:47 vapier Exp $";
 #define qpkg_usage(ret) usage(ret, QPKG_FLAGS, qpkg_long_opts, qpkg_opts_help, lookup_applet_idx("qpkg"))
 
 
@@ -142,7 +142,7 @@ int qpkg_main(int argc, char **argv)
 			restrict_chmod = 1;
 			qpkg_bindir = xstrdup(optarg);
 			if ((access(qpkg_bindir, W_OK) != 0))
-				err("%s: %s", qpkg_bindir, strerror(errno));
+				errp("%s", qpkg_bindir);
 			break;
 		COMMON_GETOPTS_CASES(qpkg)
 		}
